from kama_sdk.core.core.consts import KTEA_TYPE_SERVER
from kama_sdk.core.core.types import KteaDict

PLUGIN_ID = 'nmachine.telem'
STRATEGY_KEY = "strategy"
SVC_NAME = 'telem'


PROTOTYPE_MODE_KTEA = KteaDict(
  type=KTEA_TYPE_SERVER,
  version="1.0.0",
  uri="https://api.nmachine.io/ktea/nmachine/telem-plugin"
)



import os
from typing import List, Dict

from kama_sdk.core.core.plugin_type_defs import PluginManifest
from kama_sdk.utils.descriptor_utils import load_dir_yamls
from kama_telem_plugin.consts import PROTOTYPE_MODE_KTEA
from kama_telem_plugin.models.telem_state_helper import TelemStateHelper


def get_manifest() -> PluginManifest:
  return PluginManifest(
    id='nmachine.telem',
    publisher_identifier='nmachine',
    app_identifier='telem-plugin',
    model_descriptors=gather_model_descriptors(),
    asset_paths=[assets_path],
    model_classes=gather_custom_models(),
    virtual_kteas=[],
    prototype_mode_ktea=PROTOTYPE_MODE_KTEA
  )


def gather_custom_models():
  return [
    TelemStateHelper
  ]


def gather_model_descriptors() -> List[Dict]:
  return load_dir_yamls(descriptors_path, recursive=True)


root_dir = os.path.dirname(os.path.abspath(__file__))
descriptors_path = f'{root_dir}/descriptors'
assets_path = f'{root_dir}/assets'



import base64
import json
from typing import Dict, List, Optional, Any, Union

import requests
from k8kat.res.svc.kat_svc import KatSvc

from kama_sdk.core.core.config_man import config_man
from kama_sdk.core.telem.telem_backend import TelemBackend
from kama_sdk.utils import env_utils, utils
from kama_telem_plugin.consts import SVC_NAME, PLUGIN_ID, STRATEGY_KEY


class NMachineTelemBackend(TelemBackend):

  def is_enabled(self) -> bool:
    value = config_man.read_var(STRATEGY_KEY, space=PLUGIN_ID)
    return utils.any2bool(value)

  def is_online(self) -> bool:
    return self.collection_names() is not None

  @staticmethod
  def get_strategy() -> str:
    return config_man.read_var(STRATEGY_KEY, space=PLUGIN_ID)

  def collection_names(self) -> List[str]:
    endpoint = f"/collections/index"
    return self.do_get(endpoint, {})

  def drop_collection(self, coll_id: str):
    endpoint = f"/collections/{coll_id}/drop"
    return self.do_post(endpoint, {}, {})

  def create_record(self, coll_id: str, record: Dict):
    endpoint = f"/collections/{coll_id}/insert"
    self.do_post(endpoint, {}, {'data': record})

  def update_record(self, coll_id: str, record: Dict):
    pass

  def find_record_by_id(self, coll_id: str, record_id) -> Optional[Dict]:
    endpoint = f"/collections/{coll_id}/find/{record_id}"
    return self.do_get(endpoint, {})

  def query_collection(self, coll_id: str, query: Dict) -> List[Dict]:
    endpoint = f"/collections/{coll_id}/query"
    args = encode_query_arg(query)
    return self.do_get(endpoint, args)

  @staticmethod
  def get_svc() -> Optional[KatSvc]:
    return KatSvc.find(SVC_NAME, config_man.get_ns())

  @staticmethod
  def do_get(path: str, args: Dict) -> Optional[Union[Dict, List]]:
    if env_utils.is_in_cluster():
      url = path2in_cluster_url(path)
      return requests.get(url).json()
    else:
      if svc := NMachineTelemBackend.get_svc():
        result = svc.proxy_get(path, args)
        return parse_proxy_response(result)
      else:
        return None

  @staticmethod
  def do_post(path: str, args: Dict, bod) -> Optional[Union[Dict, List]]:
    if env_utils.is_in_cluster():
      url = path2in_cluster_url(path)
      return requests.post(url, json=bod).json()
    else:
      if svc := NMachineTelemBackend.get_svc():
        result = svc.proxy_post(path, args, bod)
        # print(result)
        return parse_proxy_response(result)
      else:
        return None


def encode_query_arg(query: Dict) -> Dict:
  if query:
    query_str = json.dumps(query or {})
    b64_enc_query = base64.b64encode(query_str.encode())
    return dict(query=b64_enc_query)
  else:
    return {}


def parse_proxy_response(result: Any) -> Optional[Union[Dict, List]]:
  if isinstance(result, dict):
    if result.get('status') < 400:
      parsed_body = result.get('body')
      if parsed_body and 'data' in parsed_body.keys():
        return parsed_body.get('data')
      else:
        return parsed_body
    else:
      return None


def path2in_cluster_url(path: str) -> str:
  base = f"http://{SVC_NAME}.{config_man.ns()}:5000"
  return f"{base}/{path}"



import os

import setuptools


def package_files(directory):
  paths = []
  for (path, directories, filenames) in os.walk(directory):
    for filename in filenames:
      paths.append(os.path.join('..', path, filename))
  return paths


descriptor_files = package_files('kama_telem_plugin/descriptors')
asset_files = package_files('kama_telem_plugin/assets')

with open("README.md", "r") as fh:
  long_description = fh.read()


setuptools.setup(
  name="kama-telem-plugin",
  version="0.0.7",
  author="NMachine",
  author_email="xavier@nmachine.io",
  description="Telemetry plugin for the kama-sdk-py",
  long_description=long_description,
  long_description_content_type="text/markdown",
  url="https://github.com/nmachine-io/kama-sdk-py",
  packages=setuptools.find_packages(exclude=['kama_sdk']),
  package_data={
    '': descriptor_files + asset_files
    # 'kama_telem_plugin': [
    #   'assets/*.*',
    #   'descriptors'
    # ]
  },
  include_package_data=True,
  install_requires=[
    'kama-sdk-py',
  ],
  classifiers=[
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
  ],
  python_requires='>=3.8'
)



from typing import Optional

from k8kat.res.svc.kat_svc import KatSvc

from kama_sdk.core.telem.telem_manager import telem_manager
from kama_sdk.model.base.model import Model
from kama_sdk.model.base.model_decorators import model_attr
from kama_telem_plugin.nmachine_telem_backend import NMachineTelemBackend


class TelemStateHelper(Model):

  @model_attr(key='is_online', cached=True)
  def is_online(self) -> bool:
    if backend := get_backend():
      return backend.is_enabled() and backend.is_online()
    else:
      return False

  @model_attr(key='is_offline', cached=True)
  def is_offline(self):
    return not self.is_online()

  @model_attr(key='is_enabled', cached=True)
  def is_enabled(self):
    if backend := get_backend():
      return backend.is_enabled() and backend.is_enabled()
    else:
      return False

  @model_attr(key='is_disabled', cached=True)
  def is_disabled(self):
    return not self.is_enabled()

  @model_attr(key='svc', cached=True)
  def get_svc(self) -> Optional[KatSvc]:
    if backend := get_backend():
      return backend.get_svc()

  @model_attr(key='strategy')
  def get_strategy(self) -> Optional[str]:
    if backend := get_backend():
      return backend.get_strategy()

  @model_attr(key='status', cached=True)
  def status(self):
    if backend := get_backend():
      if backend.is_enabled():
        if backend.is_online():
          return 'online'
        else:
          return 'offline'
      else:
        return 'disabled'
    else:
      return 'disabled'


def get_backend() -> Optional[NMachineTelemBackend]:
  backend = telem_manager.get_backend()
  is_from_this_plugin = isinstance(backend, NMachineTelemBackend)
  return backend if is_from_this_plugin else None
